/***************************************************************************
                          MiniHTTP.h  -  description
                             -------------------
    begin                : Fri May 4 2002
    copyright            : (C) 2002 by Henrik Baastrup
    email                : hba@libero.it
    home page            : http://digilander.iol.it/hba/Linux/index.html
 ***************************************************************************/

/***************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU LGPL (Lesser general Public License) 
 *   as published by the Free Software Foundation:
 *   http://www.gnu.org/copyleft/lesser.html or in the file copying.txt
 *
 ***************************************************************************/
#ifndef _MINIHTTP_H_
#define _MINIHTTP_H_

#define MINIHTTP_VERSION "1.0"

#ifdef LINUX
 #include <stdio.h>
 #include <string.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <pthread.h>
#elif defined(WIN32)
 #ifndef _WINDOWS_H_INCLUDED_
  #define _WINDOWS_H_INCLUDED_
  #include <windows.h>
 #endif
 #include <stdio.h>
 #include <string.h>
 #include <winsock.h>
#endif

#include "HttpSession.h"
#include "HttpLog.h"

#ifndef SOCKET
 #define SOCKET int
#endif

class MiniHTTP
{
	private:
		SOCKET m_soccketFd;
		HttpSession *m_firstSession, *m_lastSession;
		int m_stopMiniHTTP;
		int m_miniHttpGarbageColectorIsRunning;
		char m_homeDir[1024];
		HttpLog *m_log;
#ifdef LINUX
		pthread_mutex_t m_mutex;
		pthread_mutexattr_t m_mutex_attr;
		pthread_t  m_threadId;
		pthread_attr_t m_threadAttr;
#elif defined(WIN32)
		CRITICAL_SECTION m_mutex;
		HANDLE m_threadId;
#endif
		void (*m_getFunctionCall)(char *, HttpSession *);
		int (*m_postFunctionCall)(char *, char **, HttpSession *);
		char m_errorString[1024];
		int m_haveError;

		void cleanup();

	public:
		MiniHTTP();
		~MiniHTTP();

		int runService(short);
		void insertSession(HttpSession *);
		void deleteSession(HttpSession *);
		HttpSession *getFirstSession() {return m_firstSession;};
		int isGarbageColectorRunning() {return m_miniHttpGarbageColectorIsRunning;};
		void logWrite(int level, const char *message) {if (m_log) m_log->logWrite(level, message);};

		char *getHomeDir() {return m_homeDir;};
		void setHomeDir(char *aHomeDir) {strncpy(m_homeDir, aHomeDir, 1024); m_homeDir[1023] = '\0';};
		int getHaveError() {return m_haveError;};
		char *getErrorString() {return m_errorString;};
		void stopMiniHttp() {m_stopMiniHTTP = 1;};
		void setGetFunctionCall(void (*call)(char *aHttpBody, HttpSession *aHttpSession)) {m_getFunctionCall = call;};
		void callGetFunction(char *aHttpBody, HttpSession *aHttpSession) {if (m_getFunctionCall) m_getFunctionCall(aHttpBody, aHttpSession);};
		void setPostFunctionCall(int (*call)(char *aHttpBody, char **aResponseMsgPtr, HttpSession *aHttpSession)) {m_postFunctionCall = call;};
		int callPostFunction(char *aHttpBody, char **aResponseMsgPtr, HttpSession *aHttpSession) {if (m_postFunctionCall) return m_postFunctionCall(aHttpBody, aResponseMsgPtr, aHttpSession); return 0;};
};//MiniHTTP

#endif
